/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::IndexMemory
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <ArmarXCore/core/Component.h>

#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>

#include <RobotAPI/libraries/armem/server/plugins/ReadWritePluginUser.h>


namespace armarx
{
    /**
     * @defgroup Component-IndexMemory IndexMemory
     * @ingroup RobotAPI-Components
     * A description of the component IndexMemory.
     *
     * @class IndexMemory
     * @ingroup Component-IndexMemory
     * @brief Brief description of class IndexMemory.
     *
     * Detailed description of class IndexMemory.
     */
    class IndexMemory :
        virtual public armarx::Component,
        virtual public armem::server::ReadWritePluginUser,
        virtual public LightweightRemoteGuiComponentPluginUser
    {
    public:
        std::string getDefaultName() const override;


        // LightweightRemoteGuiComponentPluginUser interface
    public:
        void createRemoteGuiTab();
        void RemoteGui_update() override;


    protected:
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;


    private:
        struct Properties
        {
            int maxHistorySize = 1024;
        };
        Properties properties;


        struct RemoteGuiTab : RemoteGui::Client::Tab
        {
        };
        RemoteGuiTab tab;
    };
} // namespace armarx
