/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <string>

#include <RobotAPI/libraries/aron/codegeneration/codegenerator/codewriter/cpp/generator/detail/NDArrayGenerator.h>
#include <RobotAPI/libraries/aron/core/type/variant/ndarray/NDArray.h>

namespace armarx::aron::codegenerator::cpp::generator
{
    class NDArray : public detail::NDArrayGenerator<type::NDArray, NDArray>
    {

    public:
        // constructors
        NDArray(const type::NDArray&);
        virtual ~NDArray() = default;

        // virtual implementations
        std::pair<std::vector<std::pair<std::string, std::string>>, bool>
        getCtorInitializers(const std::string&) const final;
        CppBlockPtr getResetSoftBlock(const std::string& cppAccessor) const final;
        CppBlockPtr getWriteTypeBlock(const std::string& typeAccessor,
                                      const std::string& cppAccessor,
                                      const Path&,
                                      std::string& variantAccessor) const final;
        CppBlockPtr getWriteBlock(const std::string& cppAccessor,
                                  const Path&,
                                  std::string& variantAccessor) const final;
        CppBlockPtr getReadBlock(const std::string& cppAccessor,
                                 const std::string& variantAccessor) const final;
    };
} // namespace armarx::aron::codegenerator::cpp::generator
