#include "SkillDescription.h"

#include <ArmarXCore/core/ice_conversions.h>
#include <ArmarXCore/core/time/ice_conversions.h>

namespace armarx
{
    namespace skills
    {

        SkillDescription::SkillDescription(const SkillID& id,
                                           const std::string& desc,
                                           const aron::data::DictPtr& data,
                                           const armarx::core::time::Duration& timeout,
                                           const aron::type::ObjectPtr& acceptedType,
                                           const aron::type::ObjectPtr& returnType) :
            skillId(id),
            description(desc),
            rootProfileParameterization(data),
            timeout(timeout),
            acceptedType(acceptedType),
            returnType(returnType)
        {
        }

        provider::dto::SkillDescription
        SkillDescription::toProviderIce() const
        {
            provider::dto::SkillDescription ret;
            ret.acceptedType = aron::type::Object::ToAronObjectDTO(acceptedType);
            ret.returnType = aron::type::Object::ToAronObjectDTO(returnType);
            ret.description = description;
            ret.skillId = skillId.toProviderIce();
            ret.rootProfileDefaults = aron::data::Dict::ToAronDictDTO(rootProfileParameterization);

            armarx::core::time::toIce(ret.timeout, timeout);
            return ret;
        }

        manager::dto::SkillDescription
        SkillDescription::toManagerIce() const
        {
            manager::dto::SkillDescription ret;
            ret.acceptedType = aron::type::Object::ToAronObjectDTO(acceptedType);
            ret.returnType = aron::type::Object::ToAronObjectDTO(returnType);
            ret.description = description;
            ret.skillId = skillId.toManagerIce();
            ret.rootProfileDefaults = aron::data::Dict::ToAronDictDTO(rootProfileParameterization);

            armarx::core::time::toIce(ret.timeout, timeout);
            return ret;
        }

        SkillDescription
        SkillDescription::FromIce(const provider::dto::SkillDescription& i,
                                  const std::optional<ProviderID>& pid)
        {
            armarx::core::time::Duration _d;
            armarx::core::time::fromIce(i.timeout, _d);
            return SkillDescription(
                SkillID::FromIce(i.skillId, pid),
                i.description,
                armarx::aron::data::Dict::FromAronDictDTO(i.rootProfileDefaults),
                _d,
                armarx::aron::type::Object::FromAronObjectDTO(i.acceptedType),
                armarx::aron::type::Object::FromAronObjectDTO(i.returnType));
        }

        SkillDescription
        SkillDescription::FromIce(const manager::dto::SkillDescription& i)
        {
            armarx::core::time::Duration _d;
            armarx::core::time::fromIce(i.timeout, _d);
            return SkillDescription(
                SkillID::FromIce(i.skillId),
                i.description,
                armarx::aron::data::Dict::FromAronDictDTO(i.rootProfileDefaults),
                _d,
                armarx::aron::type::Object::FromAronObjectDTO(i.acceptedType),
                armarx::aron::type::Object::FromAronObjectDTO(i.returnType));
        }
    } // namespace skills
} // namespace armarx
