#pragma once

#include <string>
#include <vector>

#include <ArmarXCore/core/time/DateTime.h>
#include <ArmarXCore/core/time/ice_conversions.h>

#include <RobotAPI/interface/skills/SkillProviderInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include "SkillID.h"
#include "SkillParameterization.h"

namespace armarx
{
    namespace skills
    {
        struct SkillExecutionID
        {
            SkillID skillId;
            std::string executorName;
            armarx::core::time::DateTime executionStartedTime;

            static const constexpr char* UNKNOWN = "UNKNOWN";
            static const constexpr char* ENTER_SEPARATOR = "[";
            static const constexpr char* EXIT_SEPARATOR = "]";
            static const constexpr char* SEPARATOR = "@";

            SkillExecutionID() = delete;
            SkillExecutionID(const SkillID&,
                             const std::string& executorName,
                             const armarx::core::time::DateTime&);

            bool
            operator==(const SkillExecutionID& other) const
            {
                if (skillId != other.skillId)
                {
                    return false;
                }
                if (executionStartedTime != other.executionStartedTime)
                {
                    return false;
                }
                if (executorName != other.executorName)
                {
                    return false;
                }
                return true;
            }

            bool
            operator<(const SkillExecutionID& other) const
            {
                // We explicitly do not compare skillIds as we ONLY want to bring the executionids in some (temporal) order
                return executionStartedTime < other.executionStartedTime;
            }

            skills::manager::dto::SkillExecutionID toManagerIce() const;

            skills::provider::dto::SkillExecutionID toProviderIce() const;

            static SkillExecutionID FromIce(const skills::manager::dto::SkillExecutionID&);

            static SkillExecutionID FromIce(const skills::provider::dto::SkillExecutionID&,
                                            const std::optional<skills::ProviderID>& providerName);

            std::string toString(const std::string& prefix = "") const;
        };

    } // namespace skills
} // namespace armarx
