#include "SkillExecutionRequest.h"

namespace armarx
{
    namespace skills
    {
        SkillExecutionRequest::SkillExecutionRequest(
            const skills::SkillID& skill,
            const std::string executorName,
            const armarx::aron::data::DictPtr& params,
            const callback::dti::SkillProviderCallbackInterfacePrx& callback) :
            skillId(skill), executorName(executorName), params(params), callbackInterface(callback)
        {
        }

        manager::dto::SkillExecutionRequest
        SkillExecutionRequest::toManagerIce() const
        {
            manager::dto::SkillExecutionRequest ret;
            ret.skillId = skillId.toManagerIce();
            ret.executorName = executorName;
            ret.params = armarx::aron::data::Dict::ToAronDictDTO(params);
            return ret;
        }

        provider::dto::SkillExecutionRequest
        SkillExecutionRequest::toProviderIce() const
        {
            provider::dto::SkillExecutionRequest ret;
            ret.skillId = skillId.toProviderIce();
            ret.executorName = executorName;
            ret.params = armarx::aron::data::Dict::ToAronDictDTO(params);
            ret.callbackInterface = callbackInterface;
            return ret;
        }

        SkillExecutionRequest
        SkillExecutionRequest::FromIce(const manager::dto::SkillExecutionRequest& req)
        {
            return {skills::SkillID::FromIce(req.skillId),
                    req.executorName,
                    armarx::aron::data::Dict::FromAronDictDTO(req.params),
                    nullptr};
        }

        SkillExecutionRequest
        SkillExecutionRequest::FromIce(const provider::dto::SkillExecutionRequest& req,
                                       const std::optional<skills::ProviderID>& providerId)
        {
            return {skills::SkillID::FromIce(req.skillId, providerId),
                    req.executorName,
                    armarx::aron::data::Dict::FromAronDictDTO(req.params),
                    req.callbackInterface};
        }

    } // namespace skills
} // namespace armarx
