#include "SkillID.h"

namespace armarx
{
    namespace skills
    {
        SkillID::SkillID(const std::string& skillName) :
            providerId(std::nullopt), skillName(skillName)
        {
        }

        SkillID::SkillID(const ProviderID& providerId, const std::string& skillName) :
            providerId(providerId), skillName(skillName)
        {
            if (simox::alg::contains(providerId.providerName, NAME_SEPARATOR) ||
                simox::alg::contains(skillName, NAME_SEPARATOR))
            {
                throw error::SkillException(
                    __PRETTY_FUNCTION__,
                    std::string("A skill provider or a skill contains the blacklisted token '") +
                        NAME_SEPARATOR + "'.");
            }

            if (simox::alg::contains(providerId.providerName, PREFIX_SEPARATOR) ||
                simox::alg::contains(skillName, PREFIX_SEPARATOR))
            {
                throw error::SkillException(
                    __PRETTY_FUNCTION__,
                    std::string("A skill provider or a skill contains the blacklisted token '") +
                        PREFIX_SEPARATOR + "'.");
            }
        }

        bool
        SkillID::operator==(const SkillID& other) const
        {
            return providerId == other.providerId && skillName == other.skillName;
        }

        bool
        SkillID::operator!=(const SkillID& other) const
        {
            return not(*this == other);
        }

        bool
        SkillID::operator<(const SkillID& other) const
        {
            return toString() < other.toString();
        }

        SkillID
        SkillID::FromIce(const manager::dto::SkillID& s)
        {
            return SkillID(s.providerId.providerName, s.skillName);
        }

        SkillID
        SkillID::FromIce(const provider::dto::SkillID& s,
                         const std::optional<ProviderID>& providerId)
        {
            if (providerId.has_value())
            {
                return SkillID(*providerId, s.skillName);
            }
            return SkillID(s.skillName);
        }

        manager::dto::SkillID
        SkillID::toManagerIce() const
        {
            ARMARX_CHECK(fullySpecified());
            return {providerId->toManagerIce(), skillName};
        }

        provider::dto::SkillID
        SkillID::toProviderIce() const
        {
            return {skillName};
        }

        std::string
        SkillID::toString(const std::string& prefix) const
        {
            if (providerId.has_value())
            {
                return (prefix.empty() ? std::string("") : (prefix + PREFIX_SEPARATOR)) +
                       providerId->providerName + NAME_SEPARATOR + skillName;
            }
            return (prefix.empty() ? std::string("") : (prefix + PREFIX_SEPARATOR)) + UNKNOWN +
                   NAME_SEPARATOR + skillName;
        }
    } // namespace skills

    std::ostream&
    skills::operator<<(std::ostream& os, const SkillID& id)
    {
        return os << "'" << id.toString() << "'";
    }
} // namespace armarx
