#pragma once

#include <string>
#include <vector>

#include <SimoxUtility/algorithm/string.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>

#include "ProviderID.h"
#include "error/Exception.h"

namespace armarx
{
    namespace skills
    {
        class SkillID
        {
        public:
            static const constexpr char* PREFIX_SEPARATOR = "->";
            static const constexpr char* NAME_SEPARATOR = "/";
            static const constexpr char* UNKNOWN = "UNKNOWN";

            std::optional<ProviderID> providerId;
            std::string skillName;

            SkillID() = delete;
            SkillID(const std::string& skillName);
            SkillID(const ProviderID& providerId, const std::string& skillName);

            bool operator==(const SkillID& other) const;
            bool operator!=(const SkillID& other) const;
            bool operator<(const SkillID& other) const;

            bool
            fullySpecified() const
            {
                return skillSpecified() && providerSpecified();
            }

            bool
            skillSpecified() const
            {
                return skillName != UNKNOWN;
            }

            bool
            providerSpecified() const
            {
                if (not providerId.has_value())
                {
                    return false;
                }
                return providerId->providerName != UNKNOWN;
            }

            manager::dto::SkillID toManagerIce() const;
            provider::dto::SkillID toProviderIce() const;

            static SkillID FromIce(const manager::dto::SkillID&);
            static SkillID FromIce(const provider::dto::SkillID&,
                                   const std::optional<ProviderID>& providerId = std::nullopt);

            std::string toString(const std::string& prefix = "") const;
        };

        std::ostream& operator<<(std::ostream& os, const SkillID& id);
    } // namespace skills
} // namespace armarx
