#pragma once

#include <tuple>

#include <RobotAPI/libraries/skills/core/Skill.h>

namespace armarx
{
    namespace skills
    {
        class SkillFactory
        {
        public:
            using FunTSkill = std::function<std::unique_ptr<Skill>()>;

            SkillFactory(const FunTSkill& s) : createS(s)
            {
            }

            template <class _Skill, class... Args>
            requires isSkill<_Skill>

            static std::unique_ptr<SkillFactory>
            ForSkill(Args&&... args)

            {
                // capture params in new lambda
                // (https://stackoverflow.com/questions/47496358/c-lambdas-how-to-capture-variadic-parameter-pack-from-the-upper-scope)
                auto createS = [... args = std::forward<Args>(args)]()
                { return std::make_unique<_Skill>(std::forward<Args>(args)...); };

                auto ret = std::make_unique<SkillFactory>(createS);
                return ret;
            }

            virtual std::unique_ptr<Skill>
            createSkill(const skills::ProviderID& pid) const
            {
                auto s = createS();
                s->setProviderId(pid);
                return s;
            }

            virtual SkillDescription
            createSkillDescription(const skills::ProviderID& pid) const
            {
                auto s = createSkill(pid);
                return s->getSkillDescription();
            }


        protected:
            FunTSkill createS;
        };

        template <class T>
        concept isSkillFactory = std::is_base_of<SkillFactory, T>::value;

    } // namespace skills
} // namespace armarx
